/* HUB ISP - Program ATMEGA328P using only a USB HUB and simple circuitry
 * Copyright 2010, Paul Stoffregen <paul@pjrc.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "headers.h"


static int xfer_bit(int b)
{
	static int mosi_state = -1;
	int in;

	if ((b && mosi_state == 1) || (!b && mosi_state == 0)) {
		// MOSI is already correct
		in = read_port(4) ? 0 : 1;
		pulse_port(3);
		delay(0.001);
	} else {
		// MOSI needs to be set or cleared
		pulse_begin_port(b ? 1 : 2);
		mosi_state = (b ? 1 : 0);
		in = read_port(4) ? 0 : 1;
		pulse_port(3);
		pulse_end_port(b ? 1 : 2);
		delay(0.001);
	}
	return in;
}

static int xfer_byte(int c)
{
	int mask, n=0;

	for (mask=0x80; mask; mask >>= 1) {
		if (xfer_bit(c & mask)) n |= mask;
	}
	return n;
}

unsigned char programming_enable(void)
{
	unsigned char c;

	xfer_byte(0b10101100);
	xfer_byte(0b01010011);
	c = xfer_byte(0);
	xfer_byte(0);
	return c;
}

unsigned char read_signature_byte(unsigned char n)
{
	xfer_byte(0b00110000);
	xfer_byte(0);
	xfer_byte(n & 3);
	return xfer_byte(0);
}

void erase_chip(void)
{
	xfer_byte(0b10101100);
	xfer_byte(0b10000000);
	xfer_byte(0);
	xfer_byte(0);
	delay(0.05);
}

void write_word(unsigned char offset, unsigned char first_byte, unsigned char second_byte)
{
	xfer_byte(0b01000000);
	xfer_byte(0);
	xfer_byte(offset);
	xfer_byte(first_byte);
	xfer_byte(0b01001000);
	xfer_byte(0);
	xfer_byte(offset);
	xfer_byte(second_byte);
}

void write_page(int addr)
{
	xfer_byte(0b01001100);
	xfer_byte(addr >> 8);
	xfer_byte(addr & 255);
	xfer_byte(0);
	delay(0.01);
}

int write_fuse_lsb(unsigned char fuse)
{
	xfer_byte(0b10101100);
	xfer_byte(0b10100000);
	xfer_byte(0);
	xfer_byte(fuse);
	delay(0.01);
	xfer_byte(0b01010000);
	xfer_byte(0b00000000);
	xfer_byte(0);
	if (xfer_byte(0) != fuse) return -1;
	return 0;
}

int write_fuse_msb(unsigned char fuse)
{
	xfer_byte(0b10101100);
	xfer_byte(0b10101000);
	xfer_byte(0);
	xfer_byte(fuse);
	delay(0.01);
	xfer_byte(0b01011000);
	xfer_byte(0b00001000);
	xfer_byte(0);
	if (xfer_byte(0) != fuse) return -1;
	return 0;
}

int write_fuse_ext(unsigned char fuse)
{
	xfer_byte(0b10101100);
	xfer_byte(0b10100100);
	xfer_byte(0);
	xfer_byte(fuse);
	delay(0.01);
	xfer_byte(0b01010000);
	xfer_byte(0b00001000);
	xfer_byte(0);
	if (xfer_byte(0) != fuse) return -1;
	return 0;
}

int write_lock_bits(unsigned char lock)
{
	xfer_byte(0b10101100);
	xfer_byte(0b11100000);
	xfer_byte(0);
	xfer_byte(lock);
	delay(0.01);
	xfer_byte(0b01011000);
	xfer_byte(0);
	xfer_byte(0);
	if (xfer_byte(0) != lock) return -1;
	return 0;
}

